-------------------------------------------------------------
-- tDaqEcalLinkData : type tDaqEcalLinkData is array(integer range 0 to 6) of std_logic_vector(15 downto 0);
--   contains 7 of 16-bit ecal channel data
-------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

use work.emp_data_types.all;
use work.mprocessor_constants.all;

entity EcalDataDecoder is
port (
       --- Input Ports ---
        clk_p            : in  std_logic;
        data_in          : in lword                        := LWORD_NULL;
        --- Output Ports ---
        ecal_out         : out tDaqEcalLinkData
);
end EcalDataDecoder;

architecture rtl of EcalDataDecoder is

      type tElinkDataArray is array(integer range 0 to 13) of std_logic_vector(7 downto 0);
      constant ElinkDataArray_NULL : tElinkDataArray := (others => (others => '0'));
      
      signal elink_data : tElinkDataArray := ElinkDataArray_NULL;

			type elink_to_lword_type is array(integer range 0 to 13) of integer;
			constant elink_to_lword : elink_to_lword_type := (3,2,1,6,0,4,5,32+3,32+2,32+1,32+6,32+0,32+4,32+5);

begin

  decoder: process (clk_p)

  begin
    if rising_edge(clk_p) then
       
	    for i in 0 to 13 loop 
 	    	elink_data(i) <= elink_data(i)(6 downto 0) & data_in.data(elink_to_lword(i));
      end loop;
        
    end if;

end process decoder;

--==============================--
LinkEcalDataFormatter : for i in 0 to 6 generate
--==============================--

    ecal_out(i) <= elink_data(i+7) & elink_data(i);
  
end generate LinkEcalDataFormatter;
  
end rtl;

