-------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

use work.ipbus.all;
use work.ipbus_reg_types.all;
use work.emp_data_types.all;
use work.mprocessor_constants.all;
use work.ecal_zsf_type_pckg.all;


entity EcalDataFilter is
port (
       --- Input Ports ---
        clk_p            : in  std_logic;
        clk40            : in  std_logic;
        reset            : in  std_logic;
        data_in_start    : in  std_logic;
        data_in          : in  std_logic_vector(28 * 16 -1 downto 0);
        --- Output Ports ---
        trig_o_lat       : out std_logic_vector(5 downto 0);
        trig_o           : out std_logic_vector(31 downto 0); -- bit31 : start, bit30 : last, bit27-1 : all_valids, bit0 : valid
        --- IPBus Ports ---
        ipb_clk            : in  std_logic;
        ipb_rst            : in  std_logic;
        ipb_in             : in  ipb_wbus;
        ipb_out            : out ipb_rbus
);
end EcalDataFilter;

architecture rtl of EcalDataFilter is


  signal regs              : ipb_reg_v(32 downto 0) := (others => (others => '0'));

  signal bypass_s          : std_logic;
	signal bypass_prescale_s : std_logic_vector(15 downto 0);
  signal b_trg             : std_logic;
	signal b_counter         : std_logic_vector(15 downto 0);

  signal data_s            : std_logic_vector(28 * 16 -1 downto 0);

  signal trig_meta         : std_logic_vector(31 downto 0);
  signal trig_sync         : std_logic_vector(31 downto 0);
  
  signal zs_sc_regs        : array_32bit(0 to 31);
  signal zs_valid          : std_logic;
  signal zs_all_valids     : std_logic_vector(26 downto 0);
  signal zs_valid_lat      : std_logic_vector(5 downto 0);


  
  signal trig40            : std_logic_vector(31 downto 0);
  signal zs_valid_p        : std_logic_vector(1 downto 0);  
  signal zs_all_valids_p0  : std_logic_vector(26 downto 0);  
  signal zs_all_valids_p1  : std_logic_vector(26 downto 0);  
begin

 
  --==============================--
  ctrl_reg: entity work.ipbus_reg_v
  --==============================--
  generic map(
        N_REG    => 33
  )
  port map(
        clk       => ipb_clk,
        reset     => ipb_rst,
        ipbus_in  => ipb_in,
        ipbus_out => ipb_out,
        q         => regs,
        qmask     => (others => (others => '1')),
        stb       => open
  );

  bypass_s <= regs(0)(0);
  bypass_prescale_s <= regs(0)(16 downto 1);
  zs_sc_regs_gen : for i in 0 to 31 generate
    zs_sc_regs(i) <= regs(i+1);
  end generate;

  trig_o_lat <= zs_valid_lat when bypass_s = '0' else "000011";
  trig_o     <= trig40       when bypass_s = '0' else b_trg & b_trg & "00" & x"000000" & "000" & b_trg;

  -- trig_o     <= trig_sync    when bypass_s = '0' else x"c0000001";

  --data_in_proc : process(clk_p)
	--	variable timer : integer range 0 to 7 := 0;
  --begin
	--	if rising_edge(clk_p) then

	--		if data_in_start = '1' then
	--			timer := 0;
	--		elsif timer = 3 then
	--			data_s <= data_in;
	--		end if;

	--		if timer < 7 then
	--			timer := timer + 1;
	--		else
	--			timer := 0;
	--		end if;

	--	end if;
	--end process;

  --trig_meta   <= trig40      when rising_edge(clk_p);
  --trig_sync   <= trig_meta   when rising_edge(clk_p);
  
  trig40_proc : process(clk40)
  begin
   if reset = '1' then
     trig40 <= (others => '0');
		 b_counter <= (others => '0');
   elsif rising_edge(clk40) then

		 data_s <= data_in;

		 b_counter <= std_logic_vector(unsigned(b_counter) + 1);
		 b_trg <= '0';
		 if b_counter = std_logic_vector(unsigned(bypass_prescale_s) - 1) then
			 b_trg <= '1';
			 b_counter <= (others => '0');
	   end if;

     zs_valid_p(0) <= zs_valid;
     zs_valid_p(1) <= zs_valid_p(0);
     zs_all_valids_p0 <= zs_all_valids;
     zs_all_valids_p1 <= zs_all_valids_p0;
     trig40 <= (others => '0');
     if zs_valid_p(1) = '0' and zs_valid_p(0) = '1' then
       trig40(31) <= '1';
     end if;
     if zs_valid_p(0) = '1' and zs_valid = '0' then
       trig40(30) <= '1';
     end if;
     trig40(27 downto 0) <= zs_all_valids_p0 & zs_valid_p(0);
   end if;
  end process;
  
  --==============================--
  EcalDataZSInstance : entity work.EcalDataZSFilter
  --==============================--
  port map (
    clk                => clk40,
    rst                => reset,
    lpgbt_payload_i    => data_s,
    sc_regs_i          => zs_sc_regs,
    valid_o            => zs_valid,
    all_valids_o       => zs_all_valids,
    valid_lat_o        => zs_valid_lat
  );
  
end rtl;

