library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.emp_data_types.all;
use work.dtc_link_maps.all;
use work.dtc_constants.all;
use work.dtc_data_types.all;
use IEEE.std_logic_misc.all;
use work.mprocessor_constants.all;


entity PayloadHeaderGenerator is
    port ( 
        --- Input Ports ---
        clk_p : in std_logic;
        reset : in std_logic;
        readout_reset : in std_logic;
        header_start : in std_logic;
        module_header_in : in tCICHeaderArray(cNumberOfFEModules * cNumberOfCICs - 1 downto 0) := (others => ('0', (others => '0'), (others => '0'), (others => '0')));
        user_bits : in std_logic_vector(31 downto 0);
        --- Ouput Ports ---
        header_out : out tPayloadHeaderArray := (others => (others => '0'));
        --- Debug Ports ---
        debug_super_id : out std_logic_vector(31 downto 0) := (others => '0')
    );
end PayloadHeaderGenerator;

architecture Behavioral of PayloadHeaderGenerator is

    type tBCIDShiftRegister    is array(1 downto 0)       of std_logic_vector(12 - 1 downto 0);
    type tInputDataCountArray  is array(integer range <>) of std_logic_vector(5 downto 0);
    
    signal bcid_sr                                                        : tBCIDShiftRegister                                 := (others => (others => '0'));
    signal super_id                                                       : unsigned(32 - 1 downto 0)                          := (others => '0');

    signal packet_stub_count, packet_stub_count_previous                  : unsigned(7 downto 0)                               := (others => '0');
    signal packet_stub_counter_pointer                                    : integer                                            := 0;
    signal packet_stub_count_done                                         : std_logic                                          := '0';
    
    signal header_word : std_logic_vector(127 downto 0) := (others => '0');

begin

    --==============================--
    pBufferHeader: process(clk_p)
    --==============================--
        variable bcid             : std_logic_vector(12 - 1 downto 0)                                     := (others => '0');
    begin
        if rising_edge(clk_p) then
            if reset = '1' then
                super_id <= (others => '0');
                header_word <= (others => '0');
            else
                if header_start = '1' then
                    bcid      := std_logic_vector(module_header_in(0).bcid);
                    bcid_sr   <= bcid_sr(bcid_sr'high - 1 downto 0) & bcid;
                    if unsigned(bcid_sr(bcid_sr'high)) > unsigned(bcid_sr(bcid_sr'high - 1)) then
                        super_id <= super_id + 1;
                    end if;
                end if;
                
                -- header_word <= (others => '0');
                header_word(31 downto 0)  <= std_logic_vector(super_id);
                header_word(63 downto 32) <= user_bits;
                header_word(75 downto 64) <= bcid_sr(bcid_sr'high);
            end if;
        end if;
    end process pBufferHeader;
    

    --==============================--
    genOutput : for i in 0 to 2 - 1 generate
    --==============================--
                        
        signal packet_stub_count, packet_stub_count_previous                  : unsigned(7 downto 0)                               := (others => '0');
        signal packet_stub_counter_pointer                                    : integer                                            := 0;
        signal packet_stub_count_done                                         : std_logic                                          := '0';

    begin
        --==============================--
        pCountStubs : process(clk_p)
        --==============================--
        begin
            if rising_edge(clk_p) then
                if readout_reset = '1' then
                    packet_stub_count <= (others => '0');
                    packet_stub_count_previous <= packet_stub_count;
                    packet_stub_counter_pointer <= 0;
                    packet_stub_count_done <= '0';
                else
                    if packet_stub_count_done = '0' then
                        if packet_stub_counter_pointer = (6 - 1) * cNumberOfCICs then
                            packet_stub_count_done <= '1';
                        end if;
                        packet_stub_count <= packet_stub_count + unsigned(module_header_in((6*i) + packet_stub_counter_pointer).stub_count) + unsigned(module_header_in((6*i) + packet_stub_counter_pointer + 1).stub_count);
                        packet_stub_counter_pointer <= packet_stub_counter_pointer + cNumberOfCICs;
                    end if;
                end if;
                
                header_out(i)(75 downto 0 ) <= header_word(75 downto 0);
                header_out(i)(83 downto 76) <= std_logic_vector(packet_stub_count_previous);
            end if;
        end process pCountStubs;

    end generate ; -- genOutput
    
    debug_super_id <= std_logic_vector(super_id);

end Behavioral;
